@@ -26,7 +26,7 @@ from typing import Optional, List, Tuple, Iterable, Callable, Union
 from mpd.base import HELLO_PREFIX, ERROR_PREFIX, SUCCESS
 from mpd.base import MPDClientBase
 from mpd.base import MPDClient as SyncMPDClient
-from mpd.base import ProtocolError, ConnectionError, CommandError
+from mpd.base import ProtocolError, ConnectionError, CommandError, CommandListError
 from mpd.base import mpd_command_provider
 
 
@@ -202,7 +202,9 @@ class MPDClient(MPDClientBase):
     async def connect(self, host, port=6600, loop=None):
         if loop is not None:
             warnings.warn("loop passed into MPDClient.connect is ignored, this will become an error", DeprecationWarning)
-        if "/" in host:
+        if host.startswith("@"):
+            host = "\0" + host[1:]
+        if host.startswith("\0") or "/" in host:
             r, w = await asyncio.open_unix_connection(host)
         else:
             r, w = await asyncio.open_connection(host, port)
@@ -278,6 +280,13 @@ class MPDClient(MPDClientBase):
                     # in this case is intended, and is just what asyncio.Queue
                     # suggests for "get with timeout".
 
+                    if not self.__command_queue.empty():
+                        # A __command_queue.put() has happened after the
+                        # asyncio.wait_for() timeout but before execution of
+                        # this coroutine resumed. Looping around again will
+                        # fetch the new entry from the queue.
+                        continue
+
                     if self.__idle_failed:
                         # We could try for a more elaborate path where we now
                         # await the command queue indefinitely, but as we're
@@ -314,6 +323,18 @@ class MPDClient(MPDClientBase):
                     # through the result and terminate the connection
 
         except Exception as e:
+            # Pass exception to any pending task to terminate them. Otherwise they will hang
+            # indefinitely as we are about to disconnect.
+            try:
+                while not self.__command_queue.empty():
+                    pending_result = self.__command_queue.get_nowait()
+                    pending_result._feed_error(e)
+            except QueueEmpty:
+                # As per documentation, the queue raises this type of exception when get_nowait()
+                # is called and the queue is empty. It actually rather block on the get_nowait() call
+                # but let's leave the except just in case.
+                pass
+
             # Prevent the destruction of the pending task in the shutdown
             # function -- it's just shutting down by itself.
             self.__run_task = None
